@@ -45,7 +45,11 @@ GC_INNER signed_word GC_bytes_found = 0;
 STATIC ptr_t GC_leaked[MAX_LEAKED] = { NULL };
 STATIC unsigned GC_n_leaked = 0;
 
-GC_INNER GC_bool GC_have_errors = FALSE;
+#ifdef AO_HAVE_store
+  GC_INNER volatile AO_t GC_have_errors = 0;
+#else
+  GC_INNER GC_bool GC_have_errors = FALSE;
+#endif
 
 #if !defined(EAGER_SWEEP) && defined(ENABLE_DISCLAIM)
   STATIC void GC_reclaim_unconditionally_marked(void);
@@ -53,12 +57,12 @@ GC_INNER GC_bool GC_have_errors = FALSE;
 
 GC_INLINE void GC_add_leaked(ptr_t leaked)
 {
-#  ifndef SHORT_DBG_HDRS
-     if (GC_findleak_delay_free && !GC_check_leaked(leaked))
-       return;
-#  endif
+#   ifndef SHORT_DBG_HDRS
+      if (GC_findleak_delay_free && !GC_check_leaked(leaked))
+        return;
+#   endif
 
-    GC_have_errors = TRUE;
+    GC_SET_HAVE_ERRORS();
     if (GC_n_leaked < MAX_LEAKED) {
       GC_leaked[GC_n_leaked++] = leaked;
       /* Make sure it's not reclaimed this cycle */
@@ -81,7 +85,7 @@ GC_INNER void GC_print_all_errors(void)
         UNLOCK();
         return;
     }
-    have_errors = GC_have_errors;
+    have_errors = get_have_errors();
     printing_errors = TRUE;
     n_leaked = GC_n_leaked;
     if (n_leaked > 0) {
@@ -245,6 +249,7 @@ STATIC ptr_t GC_reclaim_uninit(struct hblk *hbp, hdr *hhdr, word sz,
     struct obj_kind *ok = &GC_obj_kinds[hhdr->hb_obj_kind];
     int (GC_CALLBACK *disclaim)(void *) = ok->ok_disclaim_proc;
 
+    GC_ASSERT(disclaim != 0);
 #   ifndef THREADS
       GC_ASSERT(sz == hhdr -> hb_sz);
 #   endif
@@ -435,10 +440,11 @@ STATIC void GC_reclaim_block(struct hblk *hbp, word report_if_found)
           /* Count can be low or one too high because we sometimes      */
           /* have to ignore decrements.  Objects can also potentially   */
           /* be repeatedly marked by each marker.                       */
-          /* Here we assume two markers, but this is extremely          */
+          /* Here we assume 3 markers at most, but this is extremely    */
           /* unlikely to fail spuriously with more.  And if it does, it */
           /* should be looked at.                                       */
-          GC_ASSERT(hhdr -> hb_n_marks <= 2 * (HBLKSIZE/sz + 1) + 16);
+          GC_ASSERT(sz != 0 && (GC_markers_m1 > 1 ? 3 : GC_markers_m1 + 1)
+                                * (HBLKSIZE/sz + 1) + 16 >= hhdr->hb_n_marks);
 #       else
           GC_ASSERT(sz * hhdr -> hb_n_marks <= HBLKSIZE);
 #       endif
@@ -659,7 +665,7 @@ GC_INNER void GC_start_reclaim(GC_bool report_if_found)
               }
             }
         } /* otherwise free list objects are marked,    */
-          /* and its safe to leave them                 */
+          /* and it's safe to leave them.               */
         BZERO(rlist, (MAXOBJGRANULES + 1) * sizeof(void *));
       }
 
